# require 'resque_admin/tasks'
# will give you the resque_admin tasks


namespace :resque_admin do
  task :setup

  desc "Start a ResqueAdmin worker"
  task :work => [ :preload, :setup ] do
    require 'resque_admin'

    begin
      worker = ResqueAdmin::Worker.new
    rescue ResqueAdmin::NoQueueError
      abort "set QUEUE env var, e.g. $ QUEUE=critical,high rake resque_admin:work"
    end

    worker.prepare
    worker.log "Starting worker #{self}"
    worker.work(ENV['INTERVAL'] || 5) # interval, will block
  end

  desc "Start multiple ResqueAdmin workers. Should only be used in dev mode."
  task :workers do
    threads = []

    if ENV['COUNT'].to_i < 1
      abort "set COUNT env var, e.g. $ COUNT=2 rake resque_admin:workers"
    end

    ENV['COUNT'].to_i.times do
      threads << Thread.new do
        system "rake resque_admin:work"
      end
    end

    threads.each { |thread| thread.join }
  end

  # Preload app files if this is Rails
  task :preload => :setup do
    if defined?(Rails)
      if Rails::VERSION::MAJOR > 3
        ActiveSupport.run_load_hooks(:before_eager_load, Rails.application)
        Rails.application.config.eager_load_namespaces.each(&:eager_load!)

      elsif Rails::VERSION::MAJOR == 3
        ActiveSupport.run_load_hooks(:before_eager_load, Rails.application)
        Rails.application.eager_load!

      elsif defined?(Rails::Initializer)
        $rails_rake_task = false
        Rails::Initializer.run :load_application_classes
      end
    end
  end

  namespace :failures do
    desc "Sort the 'failed' queue for the redis_multi_queue failure backend"
    task :sort do
      require 'resque_admin'
      require 'resque_admin/failure/redis'

      warn "Sorting #{ResqueAdmin::Failure.count} failures..."
      ResqueAdmin::Failure.each(0, ResqueAdmin::Failure.count) do |_, failure|
        data = ResqueAdmin.encode(failure)
        ResqueAdmin.redis.rpush(ResqueAdmin::Failure.failure_queue_name(failure['queue']), data)
      end
      warn "done!"
    end
  end
end
